@@ -17,7 +17,7 @@ from scanpy.plotting._utils import savefig_or_show
 import scanpy as sc
 
 from .. import logging as logg
-from ..tools.utils import getpath, importeR, get_X
+from ..tools.utils import importeR, get_X
 from ..tools import subset_tree
 from ..tools.fit import fit
 from .trajectory import trajectory
@@ -28,7 +28,7 @@ check = [type(imp) == str for imp in [Rpy2, R, rstats, rmgcv, Formula]]
 
 from ..get import modules as get_modules
 from .trajectory import remove_info
-from .utils import gen_milestones_gradients
+from .utils import gen_milestones_gradients, get_basis, is_categorical, subset_cells
 from .. import logging as logg
 from .. import settings
 
@@ -51,7 +51,7 @@ def trends(
     plot_heatmap: bool = True,
     wspace: Union[None, float] = None,
     show_segs: bool = True,
-    basis: str = "umap",
+    basis: Union[None, str] = None,
     heatmap_space: float = 0.5,
     offset_names: float = 0.15,
     fontsize: int = 9,
@@ -156,51 +156,20 @@ def trends(
 
     if plot_emb:
         adata_temp = adata.copy()
+        basis = get_basis(adata, basis)
 
     graph = adata.uns["graph"]
 
+    if is_categorical(adata, annot):
+        if annot + "_colors" not in adata.uns or len(adata.uns[annot + "_colors"]) == 1:
+            from . import palette_tools
+
+            palette_tools._set_default_colors_for_categorical_obs(adata, annot)
+
     if milestones is not None:
         adata = adata.copy()
-        dct = graph["milestones"]
-
-        leaves = list(map(lambda leave: dct[leave], milestones))
-        root = dct[root_milestone]
-        df = adata.obs.copy(deep=True)
-        edges = (
-            adata.uns["graph"]["pp_seg"][["from", "to"]]
-            .astype(str)
-            .apply(tuple, axis=1)
-            .values
-        )
-        img = igraph.Graph()
-        img.add_vertices(
-            np.unique(
-                adata.uns["graph"]["pp_seg"][["from", "to"]]
-                .values.flatten()
-                .astype(str)
-            )
-        )
-        img.add_edges(edges)
-
-        cells = np.unique(
-            np.concatenate(
-                list(
-                    map(
-                        lambda leave: getpath(
-                            img,
-                            root,
-                            adata.uns["graph"]["tips"],
-                            leave,
-                            adata.uns["graph"],
-                            df,
-                        ).index,
-                        leaves,
-                    )
-                )
-            )
-        )
-
-        adata = adata[cells]
+        seg_col = pd.Series(adata.uns["seg_colors"], index=adata.obs.seg.cat.categories)
+        adata = subset_cells(adata, root_milestone, milestones)
 
     if (features is None) & (cluster is None):
         features = adata.var_names
@@ -220,7 +189,40 @@ def trends(
         adata.layers["fitted"], index=adata.obs_names, columns=adata.var_names
     ).T.copy(deep=True)
     g = adata.obs.groupby("seg")
-    seg_order = g.apply(lambda x: np.min(x.t)).sort_values().index.tolist()
+
+    dct = graph["milestones"]
+    keys = np.array(list(dct.keys()))
+    vals = np.array(list(dct.values()))
+    edges = graph["pp_seg"][["from", "to"]].astype(str).apply(tuple, axis=1).values
+    img = igraph.Graph(directed=True)
+    img.add_vertices(vals.astype(str))
+    img.add_edges(edges)
+
+    allpaths = img.get_all_shortest_paths(
+        str(graph["root"]), to=graph["tips"].astype(str)
+    )
+
+    allpaths = np.array(allpaths, dtype=object)[
+        np.argsort(np.array([len(p) for p in allpaths]))
+    ]
+
+    order = allpaths[0]
+    for i in range(1, len(allpaths)):
+        order = order + np.array(allpaths[i])[~np.isin(allpaths[i], order)].tolist()
+
+    order = np.array(order)[1:]
+
+    order = pd.Series(graph["milestones"].keys(), index=graph["milestones"].values())[
+        np.array(img.vs["name"])[order].astype(int)
+    ]
+    to_mil = graph["pp_seg"].loc[adata.obs.seg.cat.categories.astype(int), "to"]
+    seg_order = pd.Series(range(len(adata.obs.seg.cat.categories)), index=to_mil)[
+        order[order.index.isin(to_mil)].index
+    ].values
+
+    seg_order = adata.obs.seg.cat.categories[seg_order]
+    vs2mils = pd.Series(dct.keys(), index=dct.values())
+
     cell_order = np.concatenate(
         list(
             map(
@@ -229,7 +231,6 @@ def trends(
         )
     )
     fitted = fitted.loc[:, cell_order]
-    # fitted=fitted.apply(lambda x: (x-x.mean())/x.std(),axis=1)
 
     fitted = fitted.apply(lambda x: (x - x.min()) / (x.max() - x.min()), axis=1)
 
@@ -305,9 +306,9 @@ def trends(
     if annot == "milestones":
         annot_cmap = gen_milestones_gradients(adata, seg_order)
 
-    elif type(annot) == str:
-        if len(adata.obs[annot].unique()) > 1:
-            color_key = annot + "_colors"
+    elif annot == "seg":
+        if len(adata.obs.seg.unique()) > 1:
+            color_key = "seg_colors"
             if color_key not in adata.uns or len(adata.uns[color_key]) == 1:
                 from . import palette_tools
 
@@ -323,6 +324,12 @@ def trends(
                 ),
                 index=fitted_sorted.columns,
             )
+        elif (len(adata.obs.seg.unique()) == 1) & (milestones is not None):
+            annot_cmap = pd.Series(
+                [seg_col[seg_order].values[0] for i in range(fitted_sorted.shape[1])],
+                index=fitted_sorted.columns,
+            )
+
     else:
         annot = None
 
@@ -650,6 +657,11 @@ def single_trend(
     """
     if feature is not None:
         adata = adata[:, feature].copy()
+        color_key = "seg_colors"
+        if color_key not in adata.uns:
+            from . import palette_tools
+
+            palette_tools._set_default_colors_for_categorical_obs(adata, "seg")
         if root_milestone is not None:
             reset = False
             if settings.verbosity > 2:
@@ -658,14 +670,15 @@ def single_trend(
                 reset = True
             if len(adata.uns["graph"]["tips"]) > 3:
                 adata = subset_tree(adata, root_milestone, milestones, copy=True)
+                adata.uns["seg_colors"] = [
+                    np.array(adata.uns["milestones_colors"])[
+                        pd.Series(adata.uns["graph"]["milestones"]) == t
+                    ][0]
+                    for t in adata.uns["graph"]["pp_seg"].to
+                ]
             fit(adata, [feature])
             if reset:
                 settings.verbosity = temp_verb
-        color_key = "seg_colors"
-        if color_key not in adata.uns:
-            from . import palette_tools
-
-            palette_tools._set_default_colors_for_categorical_obs(adata, "seg")
 
         Xfeature = np.array(get_X(adata, adata.obs_names, feature, layer)).ravel()
 
@@ -688,7 +701,7 @@ def single_trend(
 
         mod = get_modules(adata, root_milestone, milestones, layer, module)[
             module + "_" + branch
-        ]
+        ].loc[adata.obs_names]
         adata = sc.AnnData(
             mod.to_frame(), obs=adata.obs, obsm=adata.obsm, uns=adata.uns
         )
@@ -745,6 +758,7 @@ def single_trend(
                 c=adata.uns["seg_colors"][
                     np.argwhere(adata.obs.seg.cat.categories == s)[0][0]
                 ],
+                rasterized=True,
             )
             ax_trend.plot(
                 df.loc[df.seg == s, "t"],
@@ -781,6 +795,7 @@ def single_trend(
                 c=color_exp,
                 alpha=alpha_expr,
                 s=size_expr,
+                rasterized=True,
             )
             ax_trend.plot(
                 df.loc[df.seg == s, "t"],
