@@ -17,15 +17,16 @@ from matplotlib.colors import Normalize, hex2color, rgb2hex
 from numba import njit
 import math
 
-from .utils import is_categorical
+from .utils import is_categorical, get_basis, subset_cells
 from . import palette_tools
+from .milestones import milestones as milestones_plot
 from ..tools.graph_operations import subset_tree
 from .. import settings
 
 
 def graph(
     adata: AnnData,
-    basis: str = "umap",
+    basis: Union[None, str] = None,
     size_nodes: float = None,
     alpha_nodes: float = 1,
     linewidth: float = 2,
@@ -87,12 +88,16 @@ def graph(
 
     graph = adata.uns["graph"]
 
+    basis = get_basis(adata, basis)
+
     emb = adata.obsm[f"X_{basis}"]
 
     if "components" in kwargs:
-        cmp = np.array(kwargs["components"]) - 1
+        cmp = np.array(kwargs["components"])
+        emb = emb[:, cmp]
+    elif "dimensions" in kwargs:
+        cmp = np.array(kwargs["dimensions"])
         emb = emb[:, cmp]
-
     else:
         emb = emb[:, :2]
 
@@ -169,7 +174,7 @@ def graph(
 
 def trajectory(
     adata: AnnData,
-    basis: str = "umap",
+    basis: Union[None, str] = None,
     root_milestone: Union[str, None] = None,
     milestones: Union[str, None] = None,
     color_seg: str = "t",
@@ -252,12 +257,16 @@ def trajectory(
 
     graph = adata.uns["graph"]
 
+    basis = get_basis(adata, basis) if basis is None else basis
+
     emb = adata.obsm[f"X_{basis}"]
 
     if "components" in kwargs:
-        cmp = np.array(kwargs["components"]) - 1
+        cmp = np.array(kwargs["components"])
+        emb = emb[:, cmp]
+    elif "dimensions" in kwargs:
+        cmp = np.array(kwargs["dimensions"])
         emb = emb[:, cmp]
-
     else:
         emb = emb[:, :2]
 
@@ -286,15 +295,16 @@ def trajectory(
         else:
             sc.pl.embedding(adata, show=False, ax=ax, basis=basis, **kwargs)
 
-    c_edges = np.array([e.split("|") for e in adata.obs.edge], dtype=int)
-    cells = [any(np.isin(c_e, nodes)) for c_e in c_edges]
-
     import logging
 
     anndata_logger = logging.getLogger("anndata")
     prelog = anndata_logger.level
     anndata_logger.level = 40
-    adata_c = adata[cells, :]
+    adata_s = (
+        subset_cells(adata, root_milestone, milestones)
+        if milestones is not None
+        else adata.copy()
+    )
 
     if is_categorical(adata, color_cells):
         if (
@@ -304,21 +314,34 @@ def trajectory(
 
             palette_tools._set_default_colors_for_categorical_obs(adata, color_cells)
 
-        adata_c.uns[color_cells + "_colors"] = [
+        adata_s.uns[color_cells + "_colors"] = [
             adata.uns[color_cells + "_colors"][
                 np.argwhere(adata.obs[color_cells].cat.categories == c)[0][0]
             ]
-            for c in adata_c.obs[color_cells].cat.categories
+            for c in adata_s.obs[color_cells].cat.categories
+        ]
+    if is_categorical(adata, color_seg):
+        if (
+            color_seg + "_colors" not in adata.uns
+            or len(adata.uns[color_seg + "_colors"]) == 1
+        ):
+
+            palette_tools._set_default_colors_for_categorical_obs(adata, color_seg)
+
+        adata_s.uns[color_seg + "_colors"] = [
+            adata.uns[color_seg + "_colors"][
+                np.argwhere(adata.obs[color_seg].cat.categories == c)[0][0]
+            ]
+            for c in adata_s.obs[color_seg].cat.categories
         ]
 
+    plotter = milestones_plot if color_cells == "milestones" else sc.pl.embedding
+    if ("s" not in kwargs) & (milestones is not None):
+        kwargs["s"] = 120000 / adata.shape[0]
     if ax is None:
-        ax = sc.pl.embedding(
-            adata, color=color_cells, basis=basis, show=False, **kwargs
-        )
+        ax = plotter(adata_s, color=color_cells, basis=basis, show=False, **kwargs)
     else:
-        sc.pl.embedding(
-            adata, color=color_cells, basis=basis, ax=ax, show=False, **kwargs
-        )
+        plotter(adata_s, color=color_cells, basis=basis, ax=ax, show=False, **kwargs)
 
     anndata_logger.level = prelog
     if show_info == False and color_cells is not None:
@@ -378,12 +401,25 @@ def trajectory(
         color_mils = [
             mil_col[miles_cat == mm][0] for mm in [rev_dict[m] for m in miles_ids]
         ]
-
+    elif color_seg == "seg":
+        seg_edges = (
+            graph["pp_info"]
+            .loc[np.array(edges).ravel(), "seg"]
+            .values.reshape(-1, 2)[:, 0]
+        )
+        seg_col = pd.Series(adata.uns["seg_colors"], index=adata.obs.seg.cat.categories)
+        color_segs = [hex2color(seg_col.loc[s]) for s in seg_edges]
+        color_mils = [
+            hex2color(seg_col.loc[graph["pp_info"].loc[m].seg])
+            for m in graph["milestones"].values()
+        ]
     else:
         vals = pd.Series(
             _get_color_values(adata, color_seg, layer=layer_seg)[0],
             index=adata.obs_names,
         )
+        if not np.issubdtype(vals.dtype, np.number):
+            raise Exception("can only color numerical values")
         R = pd.DataFrame(adata.obsm["X_R"], index=adata.obs_names)
         R = R.loc[adata.obs_names]
         vals = vals[~np.isnan(vals)]
@@ -446,13 +482,16 @@ def trajectory(
                 zorder=101,
                 rasterized=True,
             )
-            c_arrow = vals[
-                (
-                    np.array([e.split("|") for e in adata.obs.edge], dtype=int)
-                    == path[mid]
-                ).sum(axis=1)
-                == 1
-            ].mean()
+            closest = (
+                np.array([e.split("|") for e in adata.obs.edge], dtype=int) == path[mid]
+            ).sum(axis=1) == 1
+            if color_seg == "seg":
+                c_arrow = seg_col.loc[adata.obs.seg.loc[closest]]
+            elif color_seg == "milestones":
+                mid_edge = np.argwhere((np.array(edges) == path[mid]).sum(axis=1))[0][0]
+                c_arrow = np.array(color_segs)[mid_edge]
+            else:
+                c_arrow = sm.to_rgba(vals[closest].mean())
             ax.quiver(
                 proj.loc[path[mid], 0],
                 proj.loc[path[mid], 1],
@@ -462,7 +501,7 @@ def trajectory(
                 headaxislength=10 * scale_path,
                 headlength=10 * scale_path,
                 units="dots",
-                color=sm.to_rgba(c_arrow),
+                color=c_arrow,
                 zorder=102,
                 rasterized=True,
             )
