@@ -207,7 +207,9 @@ def subset_tree(
     else:
         old_dct = None
 
+    adata.obs["old_seg"] = adata.obs.seg.copy()
     if "seg_colors" in adata.uns:
+        adata.uns["old_seg_colors"] = adata.uns["seg_colors"].copy()
         oldseg = adata.obs.seg.cat.categories.copy()
         oldseg_col = np.array(adata.uns["seg_colors"].copy())
     else:
@@ -231,6 +233,8 @@ def subset_tree(
 
     if "milestones_colors" in adata.uns:
         oldmilcol = np.array(adata.uns["milestones_colors"])
+    else:
+        oldmilcol = None
 
     milpath = img.get_all_shortest_paths(
         str(dct[root_milestone]), [str(dct[m]) for m in milestones]
@@ -304,7 +308,7 @@ def subset_tree(
     rename_milestones(adata, newmil)
     milsel = newmil
 
-    if "milestones_colors" in adata.uns:
+    if ("milestones_colors" in adata.uns) & (oldmilcol is not None):
         newcols = [oldmilcol[oldmil == m][0] for m in milsel]
     else:
         newcols = None
@@ -495,144 +499,6 @@ def attach_tree(
     return adata
 
 
-def extend_tip(adata, tip, use_rep, restrict_seg):
-
-    graph = adata.uns["graph"].copy()
-    selecta = adata.obsm["X_R"].argmax(axis=1) == tip
-    adata_sub = adata[selecta]
-
-    if sum(selecta) == 0:
-        return adata
-
-    early_cell = (
-        adata_sub.obs.t.idxmin() if tip == graph["root"] else adata_sub.obs.t.idxmax()
-    )
-
-    cellsel = (
-        adata.obs_names[adata.obs.seg == adata.obs.seg[early_cell]]
-        if restrict_seg
-        else adata.obs_names
-    )
-
-    ms_data = adata[cellsel].obsm[use_rep]
-    from sklearn import preprocessing
-
-    data = pd.DataFrame(
-        preprocessing.minmax_scale(ms_data),
-        index=adata[cellsel].obs_names,
-    )
-
-    #  Start cell as the nearest diffusion map boundary
-    dm_boundaries = pd.Index(set(data.idxmax()).union(data.idxmin()))
-    dists = pairwise_distances(
-        data.loc[dm_boundaries, :], data.loc[early_cell, :].values.reshape(1, -1)
-    )
-    tip_cell = pd.Series(np.ravel(dists), index=dm_boundaries).idxmin()
-
-    adata_sub = adata[selecta]
-    graph = adata.uns["graph"].copy()
-    B = graph["B"]
-    F = graph["F"]
-    g = igraph.Graph.Adjacency((B > 0).tolist(), mode="undirected")
-
-    add = np.zeros(B.shape[0]).reshape(1, -1)
-    add[:, tip] = 1
-
-    B = np.concatenate([B, add])
-
-    add = np.zeros(B.shape[0]).reshape(-1, 1)
-    add[tip] = 1
-    B = np.concatenate([B, add], axis=1)
-
-    F = np.concatenate([F, adata[tip_cell].obsm[use_rep].reshape(-1, 1)], axis=1)
-
-    tip_nodes = np.array(g.neighborhood(tip, order=0, mindist=0) + [B.shape[0] - 1])
-    R = adata.obsm["X_R"]
-    cnei = np.isin(np.argmax(R, axis=1), tip_nodes)
-
-    R[cnei, :] = 0
-    R[:, tip_nodes[:-1]] = 0
-
-    R_tip = pairwise_distances(
-        adata[cnei].obsm[use_rep], F[:, tip_nodes].T, metric=adata.uns["ppt"]["metric"]
-    )
-    process_R_cpu(R_tip, adata.uns["ppt"]["sigma"] / 100)
-    Rsum = R_tip.sum(axis=1)
-    norm_R_cpu(R_tip, Rsum)
-
-    R = np.concatenate([R, np.zeros(R.shape[0]).reshape(1, -1).T], axis=1)
-    Rsum = R.sum(axis=1)
-    norm_R_cpu(R, Rsum)
-
-    for i, c in enumerate(
-        np.argwhere(adata.obs_names.isin(adata_sub.obs_names)).ravel()
-    ):
-        for j, n in enumerate(tip_nodes):
-            R[c, n] = R_tip[i, j]
-
-    Rsum = R.sum(axis=1)
-    norm_R_cpu(R, Rsum)
-
-    g = igraph.Graph.Adjacency((B > 0).tolist(), mode="undirected")
-    tips = np.argwhere(np.array(g.degree()) == 1).flatten()
-    forks = np.argwhere(np.array(g.degree()) > 2).flatten()
-
-    adata.uns["graph"]["tips"] = tips
-    adata.uns["graph"]["forks"] = forks
-    adata.obsm["X_R"] = R
-    adata.uns["graph"]["B"] = B
-    adata.uns["graph"]["F"] = F
-
-    return adata
-
-
-def extend_tips(adata: AnnData, restrict_seg: bool = False, copy: bool = False):
-
-    """\
-    Extend the tips of the tree to avoid compression of cells on pseudotime axis.
-
-    Employ boundary detection as used by Palantir function `identify_terminal_states`.
-    the boundary cells is then converted to a prinicpal point linked to the nearest tip.
-
-    Parameters
-    ----------
-    adata
-        Annotated data matrix.
-    restrict_seg
-        Whether to detect boundaries per segment or not.
-    milestones
-        Endpoint(s) of the path.
-    copy
-        Return a copy instead of writing to adata.
-
-    Returns
-    -------
-    adata : anndata.AnnData
-        Dataset with extended tree.
-
-    """
-
-    logg.info("extending tree", reset=True)
-
-    if "t" not in adata.obs.columns:
-        raise ValueError("You need to compute pseudotime before extending tips.")
-
-    adata = adata.copy() if copy else adata
-
-    for t in adata.uns["graph"]["tips"]:
-        adata = extend_tip(adata, t, adata.uns["graph"]["use_rep"], restrict_seg)
-
-    if any(adata.obsm["X_R"].sum(axis=0) == 0) & (restrict_seg == False):
-        raise ValueError(
-            "One tip has been assigned to no cell, rerun with restrict_seg=True."
-        )
-
-    logg.info("    finished", time=True, end=" " if settings.verbosity > 2 else "\n")
-    logg.hint("tips extended")
-
-    return adata if copy else None
-
-
 def simplify(adata: AnnData, n_nodes: int = 10, copy: bool = False):
 
     """\
@@ -806,3 +672,64 @@ def getpath(adata, root_milestone, milestones, include_root=False):
         res = pd.concat([res, df], axis=0)
 
     return res
+
+
+def convert_to_soft(
+    adata, sigma: float, lam: int, n_steps: int = 1, copy: bool = False
+):
+    """\
+    Convert an hard assignment matrix to a soft one, allowing for probabilistic mapping.
+
+    Parameters
+    ----------
+    adata
+        Annotated data matrix.
+    sigma
+        Sigma parameter from SimplePPT.
+    lam
+        Lambda parameter from SimplePPT.
+    n_steps
+        Number of steps to run the solving of R and F
+    copy
+        Return a copy instead of writing to adata.
+
+    Returns
+    -------
+    adata : anndata.AnnData
+        dataset if `copy=True` it returns or else updated these fields to `adata`:
+
+        `.uns['graph']['R']`
+            converted from hard to soft assignment matrix.
+        `.uns['graph']['F']`
+            solved from newly converted soft assignment matrix.
+
+    """
+
+    logg.info("Converting R into soft assignment matrix", reset=True)
+
+    adata = adata.copy() if copy else adata
+
+    graph = adata.uns["graph"]
+    F = graph["F"]
+    B = graph["B"]
+    X = adata.obsm[graph["use_rep"]][:, : graph["ndims_rep"]]
+
+    for i in range(n_steps):
+        R = pairwise_distances(X, F.T, metric="euclidean")
+        process_R_cpu(R, sigma)
+        Rsum = R.sum(axis=1)
+        norm_R_cpu(R, Rsum)
+        D = (np.identity(B.shape[0])) * np.array(B.sum(axis=0))
+        L = D - B
+        M = L * lam + np.identity(R.shape[1]) * np.array(R.sum(axis=0))
+        F = np.linalg.solve(M.T, (np.dot(X.T, R)).T).T
+
+    adata.obsm["X_R"] = R
+    adata.uns["graph"]["F"] = F
+
+    logg.info("    finished", time=True, end=" " if settings.verbosity > 2 else "\n")
+    logg.hint(
+        "updated \n"
+        "    .obsm['X_R'] converted soft assignment of cells to principal points.\n"
+        "    .uns['graph']['F'] coordinates of principal points in representation space."
+    )
